@@ -22,6 +22,11 @@ has required =>
   isa           => Bool,
   default       => false;
 
+has weak_ref =>
+  is            => rw,
+  isa           => Bool,
+  default       => false;
+
 has is =>
   is            => rw,
   isa           => Enum[ ro, rw, rwp, 'lazy', 'bare' ],
@@ -40,10 +45,14 @@ has isa =>
   init_arg      => 'isa';
 
 has type =>
-  init_arg      => undef,
   is            => 'lazy',
   isa           => Object|Undef,
-  builder       => 1;
+  builder       => true;
+
+has coerce =>
+  is            => 'rw',
+  isa           => Bool,
+  default       => false;
 
 has default =>
   is            => rw,
@@ -65,7 +74,6 @@ has coderef_default_variable =>
       return sprintf '$__%s_DEFAULT__', $self->name;
   };
 
-# Not actually supported yet
 has builder =>
     is            => rw,
     isa           => Str->where('length($_) > 0') | CodeRef | Undef,
@@ -179,12 +187,35 @@ sub _build_type {
     my $isa = $self->_isa
         or return undef;
 
+    state $type_registry = do {
+        require Type::Registry;
+        my $reg = 'Type::Registry'->for_me;
+        $reg->add_types('Types::Standard');
+        $reg->add_types('Types::Common::Numeric');
+        $reg->add_types('Types::Common::String');
+        $reg;
+    };
+
     require Type::Utils;
-    my $type = Type::Utils::dwim_type( $isa, fallback => [ 'make_class_type' ] );
+    my $type = Type::Utils::dwim_type(
+        $isa,
+        fallback => [ 'make_class_type' ],
+        for => __PACKAGE__,
+    );
+
+    $type
+        or croak sprintf 'Type %s cannot be found', $isa;
 
     $type->can_be_inlined
         or croak sprintf 'Type %s cannot be inlined', $type->display_name;
 
+    if ( $self->coerce ) {
+        $type->has_coercion
+            or carp sprintf 'Type %s has no coercions', $type->display_name;
+        $type->coercion->can_be_inlined
+            or carp sprintf 'Coercion to type %s cannot be inlined', $type->display_name;
+    }
+
     return $type;
 }
 
@@ -226,6 +257,15 @@ sub _empty {
     return ';';
 }
 
+sub _compile_coercion {
+    my ( $self, $expression ) = @_;
+    if ( $self->coerce and my $type = $self->type ) {
+        return sprintf 'do { my $to_coerce = %s; %s }',
+            $expression, $type->coercion->inline_coercion( '$to_coerce' );
+    }
+    return $expression;
+}
+
 sub _compile_checked_default {
     my ( $self, $selfvar ) = @_;
 
@@ -234,6 +274,10 @@ sub _compile_checked_default {
 
     local $Type::Tiny::AvoidCallbacks = 1;
 
+    if ( $self->coerce ) {
+        $default = $self->_compile_coercion( $default );
+    }
+
     return sprintf 'do { my $default_value = %s; %s or do { require Carp; Carp::croak(q[Type check failed in default: %s should be %s]) }; $default_value }',
         $default, $type->inline_check('$default_value'), $self->name, $type->display_name;
 }
@@ -269,13 +313,22 @@ sub compile_init {
             $argvar, $init_arg;
         if ( my $type = $self->type ) {
             local $Type::Tiny::AvoidCallbacks = 1;
-            $code .= sprintf '%s or do { require Carp; Carp::croak(q[Type check failed in constructor: %s should be %s]) }; ',
-                $type->inline_check(sprintf '%s->{q[%s]}', $argvar, $init_arg),
+            my $valuevar = sprintf '%s->{q[%s]}', $argvar, $init_arg;
+            if ( $self->coerce ) {
+                $code .= sprintf 'my $value = %s; ', $self->_compile_coercion( $valuevar );
+                $valuevar = '$value';
+            }
+            $code .= sprintf '%s or require Carp && Carp::croak(q[Type check failed in constructor: %s should be %s]); ',
+                $type->inline_check( $valuevar ),
                 $self->init_arg,
                 $type->display_name;
+            $code .= sprintf '%s->{q[%s]} = %s; delete %s->{q[%s]}; ',
+                $selfvar, $self->name, $valuevar, $argvar, $init_arg;
+        }
+        else {
+            $code .= sprintf '%s->{q[%s]} = delete %s->{q[%s]}; ',
+                $selfvar, $self->name, $argvar, $init_arg;
         }
-        $code .= sprintf '%s->{q[%s]} = delete %s->{q[%s]}; ',
-            $selfvar, $self->name, $argvar, $init_arg;
         $code .= ' }';
     }
 
@@ -287,8 +340,10 @@ sub compile_init {
         else {
             $code .= 'do { ';
         }
+        $code .= sprintf 'my $value = %s; ',
+            $self->_compile_checked_default( $selfvar );
         $code .= sprintf '%s->{q[%s]} = %s; ',
-            $selfvar, $self->name, $self->_compile_checked_default( $selfvar );
+            $selfvar, $self->name, '$value';
         $code .= ' }';
     }
     elsif ( defined $init_arg
@@ -298,144 +353,150 @@ sub compile_init {
             $init_arg;
     }
 
+    if ( $self->weak_ref ) {
+      $code .= sprintf ' require Scalar::Util && Scalar::Util::weaken(%s->{q[%s]});',
+         $selfvar, $self->name;
+    }
+
     return $code;
 }
 
-sub compile {
-    my $self = shift;
-
-    return sprintf <<'CODE', $self->_compile_xs, $self->_compile_perl;
-if( !$ENV{MITE_PURE_PERL} && eval { require Class::XSAccessor } ) {
-%s
-}
-else {
-%s
-}
-CODE
-}
-
-my %code_template = (
+my %code_template;
+%code_template = (
     reader => sub {
         my $self = shift;
+        my %arg = @_;
         my $slot_name = $self->name;
         my $code = sprintf '$_[0]->{q[%s]}', $slot_name;
         if ( $self->lazy ) {
             $code = sprintf '( exists($_[0]{q[%s]}) ? $_[0]{q[%s]} : ( $_[0]{q[%s]} = %s ) )',
                 $slot_name, $slot_name, $slot_name, $self->_compile_checked_default( '$_[0]' );
         }
-        $code = sprintf '@_ > 1 ? require Carp && Carp::croak("%s is a read-only attribute of @{[ref $_[0]]}") : %s',
-            $slot_name, $code;
+        unless ( $arg{no_croak} ) {
+            $code = sprintf '@_ > 1 ? require Carp && Carp::croak("%s is a read-only attribute of @{[ref $_[0]]}") : %s',
+                $slot_name, $code;
+        }
         return $code;
     },
     writer => sub {
         my $self = shift;
+        my %arg = @_;
         my $slot_name = $self->name;
-        my $code = sprintf '$_[0]->{ q[%s] } = $_[1];', $slot_name;
+        my $code = '';
         if ( my $type = $self->type ) {
             local $Type::Tiny::AvoidCallbacks = 1;
-            $code = sprintf '%s or do { require Carp; Carp::croak(q[Type check failed in writer: value should be %s]) }; %s',
-                $type->inline_check('$_[1]'), $type->display_name, $code;
+            my $valuevar = '$_[1]';
+            if ( $self->coerce ) {
+                $code .= sprintf 'my $value = %s; ', $self->_compile_coercion($valuevar);
+                $valuevar = '$value';
+            }
+            $code .= sprintf '%s or require Carp && Carp::croak(q[Type check failed in %s: value should be %s]); $_[0]{q[%s]} = %s;',
+                $type->inline_check($valuevar), $arg{label}//'writer', $type->display_name, $slot_name, $valuevar;
         }
+        else {
+            $code = sprintf '$_[0]->{q[%s]} = $_[1];', $slot_name;
+        }
+        if ( $self->weak_ref ) {
+            $code .= sprintf ' require Scalar::Util && Scalar::Util::weaken($_[0]->{q[%s]});',
+                $slot_name;
+        }
+        $code .= ' $_[0];';
         return $code;
     },
     accessor => sub {
         my $self = shift;
-        my $slot_name = $self->name;
-        my $code = sprintf '$_[0]->{q[%s]}', $slot_name;
-        if ( $self->lazy ) {
-            $code = sprintf '( exists($_[0]{q[%s]}) ? $_[0]{q[%s]} : ( $_[0]{q[%s]} = %s ) )',
-                $slot_name, $slot_name, $slot_name, $self->_compile_checked_default( '$_[0]' );
+        my %arg = @_;
+        my @parts = (
+            $code_template{writer}->( $self, label => 'accessor' ),
+            $code_template{reader}->( $self, no_croak => true ),
+        );
+        for my $i ( 0 .. 1 ) {
+            $parts[$i] = $parts[$i] =~ /\;/
+                ? "do { $parts[$i] }"
+                : "( $parts[$i] )"
         }
-        $code = sprintf '@_ > 1 ? ( $_[0]->{ q[%s] } = $_[1] ) : %s',
-            $slot_name, $code;
-        if ( my $type = $self->type ) {
-            local $Type::Tiny::AvoidCallbacks = 1;
-            $code = sprintf '@_==1 or %s or do { require Carp; Carp::croak(q[Type check failed in accessor: value should be %s]) }; %s',
-                $type->inline_check('$_[1]'), $type->display_name, $code;
-        }
-        return $code;
+        sprintf '@_ > 1 ? %s : %s', @parts;
     },
     clearer => sub {
         my $slot_name = shift->name;
-        sprintf 'delete $_[0]->{ q[%s] }; $_[0];',
+        my %arg = @_;
+        sprintf 'delete $_[0]->{q[%s]}; $_[0];',
             $slot_name;
     },
     predicate => sub {
         my $slot_name = shift->name;
-        sprintf 'exists $_[0]->{ q[%s] };',
+        sprintf 'exists $_[0]->{q[%s]}',
             $slot_name;
     },
 );
 
-sub _compile_xs {
+sub compile {
     my $self = shift;
+    my %args = @_;
 
+    my $xs_condition = $args{xs_condition}
+        || '!$ENV{MITE_PURE_PERL} && eval { require Class::XSAccessor; Class::XSAccessor->VERSION("1.19") }';
     my $slot_name = $self->name;
 
-    my %options = (
+    my %xs_option_name = (
         reader    => 'getters',
         writer    => 'setters',
         accessor  => 'accessors',
         predicate => 'exists_predicates',
     );
 
-    my %use_xs = (
-        'reader' => 1,
-        'writer' => 1,
-        'accessor' => 1,
-        'predicate' => 1,
-    );
-    my %use_pp = (
-        'clearer' => 1,
-    );
+    my %want_xs;
+    my %want_pp;
+    my %method_name;
 
-    if ( $self->type ) {
-        delete $use_xs{writer};
-        delete $use_xs{accessor};
-        $use_pp{writer} = $use_pp{accessor} = 1;
+    for my $property ( keys %code_template ) {
+        my $method_name = $self->$property;
+        next unless defined $method_name;
+
+        $method_name{$property} = $method_name;
+        if ( $xs_option_name{$property} ) {
+            $want_xs{$property} = 1;
+        }
+        $want_pp{$property} = 1;
     }
 
+    # Class::XSAccessor can't do type checks or handle weaken
+    if ( $self->type or $self->weak_ref ) {
+        delete $want_xs{writer};
+        delete $want_xs{accessor};
+    }
+
+    # Class::XSAccessor can't do lazy builders checks
     if ( $self->lazy ) {
-        delete $use_xs{reader};
-        delete $use_xs{accessor};
-        $use_pp{reader} = $use_pp{accessor} = 1;
+        delete $want_xs{reader};
+        delete $want_xs{accessor};
     }
 
-    my $xs = 0;
-    my $code = "Class::XSAccessor->import(\n";
-    for my $property ( sort keys %use_xs ) {
-        my $method_name = $self->$property;
-        next unless defined $method_name;
-        my $option = $options{$property};
-        $code .= "    $option => { q[$method_name] => q[$slot_name] },\n";
-        $xs++;
+    my $code = "# Accessors for $slot_name\n";
+    if ( keys %want_xs ) {
+        $code .= "if ( $xs_condition ) {\n";
+        $code .= "    Class::XSAccessor->import(\n";
+        $code .= "        chained => 1,\n";
+        for my $property ( sort keys %want_xs ) {
+            $code .= "        $xs_option_name{$property} => { q[$method_name{$property}] => q[$slot_name] },\n";
+        }
+        $code .= "    );\n";
+        $code .= "}\n";
+        $code .= "else {\n";
+        for my $property ( sort keys %want_xs ) {
+            $code .= sprintf '    *%s = sub { %s };' . "\n",
+                $method_name{$property}, $code_template{$property}->($self);
+            delete $want_pp{$property};
+        }
+        $code .= "}\n";
     }
-    $code .= ");\n";
-    $code = '' unless $xs;
 
-    for my $property ( sort keys %use_pp ) {
-        my $method_name = $self->$property;
-        next unless defined $method_name;
+    for my $property ( sort keys %want_pp ) {
         $code .= sprintf '*%s = sub { %s };' . "\n",
-            $method_name, $code_template{$property}->($self);
+            $method_name{$property}, $code_template{$property}->($self);
     }
 
-    return $code;
-}
-
-sub _compile_perl {
-    my $self = shift;
-
-    my $slot_name = $self->name;
-
-    my $code = '';
-
-    for my $property ( 'reader', 'writer', 'accessor', 'predicate', 'clearer' ) {
-        my $method_name = $self->$property;
-        next unless defined $method_name;
-        $code .= sprintf "    *%s = sub { %s };\n",
-            $method_name, $code_template{$property}->($self);
-    }
+    $code .= "\n";
 
     return $code;
 }
