@@ -252,7 +252,7 @@ sub _compile_bless {
 sub _compile_strict_constructor {
     my $self = shift;
 
-    return 'keys %$args and do { require Carp; Carp::croak("Unexpected keys in constructor: " . join(q[, ], sort keys %$args)) };';
+    return 'keys %$args and require Carp and Carp::croak("Unexpected keys in constructor: " . join(q[, ], sort keys %$args));';
 }
 
 sub _compile_new {
@@ -289,9 +289,12 @@ sub _compile_init_attributes {
 sub _compile_attribute_accessors {
     my $self = shift;
 
-    my $code = '';
-    for my $attribute (values %{$self->all_attributes}) {
-        $code .= $attribute->compile;
+    my $attributes = $self->all_attributes;
+    keys %$attributes or return '';
+
+    my $code = 'my $__XS = !$ENV{MITE_PURE_PERL} && eval { require Class::XSAccessor; Class::XSAccessor->VERSION("1.19") };' . "\n\n";
+    for my $name ( sort keys %$attributes ) {
+        $code .= $attributes->{$name}->compile( xs_condition => '$__XS' );
     }
 
     return $code;
